/**************************************************************************
* gl-test.cpp  OpenGL - Cube texturé
* Auteur : C.Turrier - 27 décembre 2020
* Compilation :
* g++ gl-cube.cpp -o test -lglut -lGLU -lGL
* gcc gl-cube.cpp -o test -lglut -lGLU -lGL
* Execution: ./test
* Fonctionnement:
* touches flèches G et D - rotation du cube autour de l'axe y
* touches flèches H et B - rotation du cube autour de l'axe x
* touche R - raz position du cube
*****************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <GL/freeglut.h> 
/*********************************************************************************
 * VARIABLES GLOBALES
 *********************************************************************************/
//identifiants des textures
GLuint id1, id2, id3, id4,id5, id6; 
//angle par rapport aux axes y et x
float a=0.0, b=0.0;
//tableau pour charger les octets rgba 
unsigned char *t;
//demi largeur du cube
float L=10;
/*********************************************************************************
 * load_urgba_tiff_file()
 * Cette fonction permet de charger le contenu d'un fichier tiff rgba non compressé dans une table t en memoire.
 * A l'issue de l'operation, la table t contient les w*h*4 pixels rgba, soit un total de 4+4*w*h octets 
 * Le parametre info (valeur 1 ou 0) permet d'afficher ou non un compte rendu, a l'issue de l'operation
 * Exemple :
 * unsigned char *t=NULL;
 * t=load_urgba_tiff_file("essai01.tiff",1);
 *********************************************************************************/
unsigned char * load_urgba_tiff_file (char * filename, unsigned int w, unsigned int h, int info)
{
FILE *pf; unsigned char * mem; unsigned char * t; 
unsigned int nbytes, k;
//ouverture du fichier et calcul de sa taille en octets
pf = fopen(filename, "rb");
if( pf == NULL ){printf( "Erreur ! fichier %s inexistant\n",filename ); return NULL;}
//positionnement a la fin du fichier
fseek(pf, 0, SEEK_END);
nbytes = ftell(pf); 
//positionnement au debut du fichier
rewind(pf);  
if (nbytes<w*h+8) {puts("erreur ! taille fichier tiff"); return NULL;}
//ecriture des octets du fichier dans mem
mem = (unsigned char *)malloc((nbytes+1)*sizeof(unsigned char));
fread(mem, nbytes, 1, pf); fclose(pf);	
// Ecriture des w*h*4 valeurs de pixels rgba dans t
t = (unsigned char *)malloc((4*w*h + 1)*sizeof(unsigned char));	
for (k=0; k<w*h*4; k++){t[k]=mem[k+8];}
//informations
if (info==1){printf("%s ", filename);printf("chargé: %u octets rgba\n",k);}
free(mem); 
return t;
}
//
/*********************************************************************************
 * load_textures_files()
 * Cette fonction permet de charger en memoire 6 textures 
 * de format tiff rgba non compressé placées dans le repertoire du programme
 * A l'issue de l'operation, les textures chargées en mémoire sont
 * fichier texture1.tiff -> identifiant1
 * fichier texture2.tiff -> identifiant2
 * ...
 * fichier texture6.tiff -> identifiant6
  *********************************************************************************/
void load_textures_files()
{
glGenTextures(1, &id1);glGenTextures(1, &id2);glGenTextures(1, &id3);
glGenTextures(1, &id4);glGenTextures(1, &id5);glGenTextures(1, &id6);
//texture1
t= load_urgba_tiff_file((char*)"texture1.tiff", 512,512, 1); if (t==NULL){return ;}
glBindTexture(GL_TEXTURE_2D, id1);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, t);
//parametres pour GL_TEXTURE_WRAP_S et T (c'est a dire pour u et v)
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,  GL_CLAMP_TO_BORDER);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,  GL_CLAMP_TO_BORDER);
//si la texture a une taille différente de la surface a texturer, opengl effectue un compromis
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
//
//texture2
t= load_urgba_tiff_file((char*)"texture2.tiff", 512,512, 1); if (t==NULL){return ;}
glBindTexture(GL_TEXTURE_2D, id2);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, t);
//parametres pour GL_TEXTURE_WRAP_S et T (c'est a dire pour u et v)
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,  GL_CLAMP_TO_BORDER);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,  GL_CLAMP_TO_BORDER);
//si la texture a une taille différente de la surface a texturer, opengl effectue un compromis
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
//
//texture3
t= load_urgba_tiff_file((char*)"texture3.tiff", 512,512, 1); if (t==NULL){return ;}
glBindTexture(GL_TEXTURE_2D, id3);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, t);
//parametres pour GL_TEXTURE_WRAP_S et T (c'est a dire pour u et v)
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,  GL_CLAMP_TO_BORDER);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,  GL_CLAMP_TO_BORDER);
//si la texture a une taille différente de la surface a texturer, opengl effectue un compromis
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
//
//texture4
t= load_urgba_tiff_file((char*)"texture4.tiff", 512,512, 1); if (t==NULL){return ;}
glBindTexture(GL_TEXTURE_2D, id4);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, t);
//parametres pour GL_TEXTURE_WRAP_S et T (c'est a dire pour u et v)
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
//si la texture a une taille différente de la surface a texturer, opengl effectue un compromis
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
//texture5
t= load_urgba_tiff_file((char*)"texture5.tiff", 512,512, 1); if (t==NULL){return ;}
glBindTexture(GL_TEXTURE_2D, id5);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, t);
//parametres pour GL_TEXTURE_WRAP_S et T (c'est a dire pour u et v)
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
//si la texture a une taille différente de la surface a texturer, opengl effectue un compromis
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
//texture6
t= load_urgba_tiff_file((char*)"texture6.tiff", 512,512, 1); if (t==NULL){return ;}
glBindTexture(GL_TEXTURE_2D, id6);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, t);
//parametres pour GL_TEXTURE_WRAP_S et T (c'est a dire pour u et v)
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
//si la texture a une taille différente de la surface a texturer, opengl effectue un compromis
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
//
glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
glEnable(GL_TEXTURE_2D);
}
// 
void initialise(void)
{
glClearColor(0.0, 0.0, 0.0, 1.0);
load_textures_files();
glEnable(GL_DEPTH_TEST);
glDepthFunc(GL_LESS);
}
//
void affiche(void)
{
glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
glLoadIdentity();
gluLookAt(0.0, 0.0, 50.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
glRotatef(a, 0.0, 1.0, 0.0);
glRotatef(b, 1.0, 0.0, 0.0);
//face1 du cube
glBindTexture(GL_TEXTURE_2D, id1);
glBegin(GL_POLYGON);
glTexCoord2f(0.0, 0.0); glVertex3f(-L, -L, L);
glTexCoord2f(1.0, 0.0); glVertex3f(L, -L, L);
glTexCoord2f(1.0, 1.0); glVertex3f(L, L, L);
glTexCoord2f(0.0, 1.0); glVertex3f(-L, L, L);
glEnd();
//face2 du cube
glBindTexture(GL_TEXTURE_2D, id2);
glBegin(GL_POLYGON);
glTexCoord2f(0.0, 0.0); glVertex3f(+L, -L, +L);
glTexCoord2f(1.0, 0.0); glVertex3f(+L, -L, -L);
glTexCoord2f(1.0, 1.0); glVertex3f(+L, +L, -L);
glTexCoord2f(0.0, 1.0); glVertex3f(+L, +L, +L);
glEnd();
//face3 du cube
glBindTexture(GL_TEXTURE_2D, id3);
glBegin(GL_POLYGON);
glTexCoord2f(0.0, 0.0); glVertex3f(L, -L, -L);
glTexCoord2f(1.0, 0.0); glVertex3f(-L, -L, -L);
glTexCoord2f(1.0, 1.0); glVertex3f(-L, L, -L);
glTexCoord2f(0.0, 1.0); glVertex3f(L, L, -L);
glEnd();
//face4 du cube
glBindTexture(GL_TEXTURE_2D, id4);
glBegin(GL_POLYGON);
glTexCoord2f(0.0, 0.0); glVertex3f(-L, -L, -L);
glTexCoord2f(1.0, 0.0); glVertex3f(-L, -L, +L);
glTexCoord2f(1.0, 1.0); glVertex3f(-L, +L, +L);
glTexCoord2f(0.0, 1.0); glVertex3f(-L, +L, -L);
glEnd();
//face5 du cube
glBindTexture(GL_TEXTURE_2D, id5);
glBegin(GL_POLYGON);
glTexCoord2f(0.0, 0.0); glVertex3f(-L, +L, +L);
glTexCoord2f(1.0, 0.0); glVertex3f(+L, +L, +L);
glTexCoord2f(1.0, 1.0); glVertex3f(+L, +L, -L);
glTexCoord2f(0.0, 1.0); glVertex3f(-L, +L, -L);
glEnd();
//face6 du cube
glBindTexture(GL_TEXTURE_2D, id6);
glBegin(GL_POLYGON);
glTexCoord2f(0.0, 0.0); glVertex3f(-L, -L, -L);
glTexCoord2f(1.0, 0.0); glVertex3f(+L, -L, -L);
glTexCoord2f(1.0, 1.0); glVertex3f(+L, -L, +L);
glTexCoord2f(0.0, 1.0); glVertex3f(-L, -L, +L);
glEnd();

glutSwapBuffers();
}
//
void dimensionne(int w, int h)
{
glViewport(0, 0, w, h);
glMatrixMode(GL_PROJECTION);
glLoadIdentity();
//-------------CARACTERISTIQUES DE LA CAMERA------------------
gluPerspective(45.0, w/h, 1.0, 300.0);
//-----------------------------------------------------------
glMatrixMode(GL_MODELVIEW);
glLoadIdentity();
}
//
void clavier(unsigned char touche, int x, int y)
{
switch (touche){
case 27: exit(0); break;
case 'r': a= 0.0; b=0.0; glutPostRedisplay(); break;
case 'R': a= 0.0; b=0.0; glutPostRedisplay(); break;
default:
break;
}}
//
void fleches(int touche, int x, int y)
{
if (touche==GLUT_KEY_LEFT){a=a-5.0;if (a<0.0) {a=a+360.0;}}
if (touche==GLUT_KEY_RIGHT){a=a+5.0;if(a>0.0) {a=a-360.0;}}
if (touche==GLUT_KEY_UP){b=b-5.0;if (b<0.0) {b=b+360.0;}}
if (touche==GLUT_KEY_DOWN){b=b+5.0;if(b>0.0) {b=b-360.0;}}
glutPostRedisplay();
}
//
//Programme principal
int main(int argc, char* argv[])
{
glutInit(&argc, argv);
glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
glutInitWindowSize(600, 600);
glutInitWindowPosition(100, 100);
glutCreateWindow("Test OpenGL");
glutDisplayFunc(affiche);
glutReshapeFunc(dimensionne);
glutKeyboardFunc(clavier);
glutSpecialFunc(fleches);
initialise();
glutMainLoop();
free(t);
return 0;
}
